﻿using System;

namespace ELTE.Basics.NumberTypes
{
    /// <summary>
    /// Komplex szám típusa
    /// </summary>
    class Complex : IRealCompatible
    {
        private Double _Real;
        private Double _Imaginary;

        /// <summary>
        /// Valós rész lekérdezése
        /// </summary>
        public Double Real
        {
            get { return _Real; }
        }

        /// <summary>
        /// Képzetes rész lekérdezése
        /// </summary>
        public Double Imaginary
        {
            get { return _Imaginary; }
        }

        /// <summary>
        /// Komplex szám létrehozása
        /// </summary>
        public Complex()
        {
            _Real = 0;
            _Imaginary = 0;
        }

        /// <summary>
        /// Komplex szám létrehozása
        /// </summary>
        /// <param name="real">valós rész</param>
        public Complex(Double real)
        {
            _Real = real;
            _Imaginary = 0;
        }

        /// <summary>
        /// Komplex szám létrehozása
        /// </summary>
        /// <param name="real">valós rész</param>
        /// <param name="imaginary">képzetes rész</param>
        public Complex(Double real, Double imaginary)
        {
            _Real = real;
            _Imaginary = imaginary;
        }

        /// <summary>
        /// Összeadás komplex számok között
        /// </summary>
        /// <param name="a">Első operandus</param>
        /// <param name="b">Második operandus</param>
        /// <returns>Összeadás eredménye</returns>
        public static Complex operator +(Complex a, Complex b)
        {
            return new Complex(a._Real + b._Real, a._Imaginary + b._Imaginary); 
        }

        /// <summary>
        /// Összeadás komplex és valós számok között
        /// </summary>
        /// <param name="a">Első operandus</param>
        /// <param name="b">Második operandus</param>
        /// <returns>Összeadás eredménye</returns>
        public static Complex operator +(Complex a, Double b)
        {
            return new Complex(a._Real + b, a._Imaginary); 
        }

        /// <summary>
        /// Összeadás valós és komplex számok között
        /// </summary>
        /// <param name="a">Első operandus</param>
        /// <param name="b">Második operandus</param>
        /// <returns>Összeadás eredménye</returns>
        public static Complex operator +(Double a, Complex b)
        {
            return new Complex(a + b._Real, b._Imaginary); 
        }

        /// <summary>
        /// Konjugálás
        /// </summary>
        /// <returns>Konjugált komplex szám</returns>
        public Complex Conjugation()
        {
            return new Complex(_Real, -_Imaginary);
        }

        /// <summary>
        /// Kiírás szövegként
        /// </summary>
        /// <returns>Komplex szám szövegszerű leírása</returns>
        public override string ToString()
        {
            if (_Imaginary == 0)
                return _Real.ToString();
            else if (_Imaginary > 0)
                return _Real.ToString() + "+" + _Imaginary.ToString() + "i";
            else
                return _Real.ToString() + _Imaginary.ToString() + "i";
        }

        #region IRealCompatible Members

        public Double ToDouble()
        {
            return _Real;
        }

        public void FromDouble(Double val)
        {
            _Real = val;
            _Imaginary = 0;
        }

        #endregion
    }
}
